@@ -52,8 +52,8 @@ namespace MonoDevelop.Debugger.Gdb
 		GdbCommandResult lastResult;
 		bool running;
 		Thread thread;
-		int currentThread = -1;
-		int activeThread = -1;
+		long currentThread = -1;
+		long activeThread = -1;
 		bool isMonoProcess;
 		string currentProcessName;
 		List<string> tempVariableObjects = new List<string> ();
@@ -108,10 +108,10 @@ namespace MonoDevelop.Debugger.Gdb
 				StartGdb ();
 				
 				// Initialize the terminal
-				RunCommand ("-inferior-tty-set", tty);
-				RunCommand ("-file-exec-and-symbols", startInfo.Command);
+				RunCommand ("-inferior-tty-set", Escape (tty));
+				RunCommand ("-file-exec-and-symbols", Escape (startInfo.Command));
 
-				RunCommand ("-environment-directory", startInfo.WorkingDirectory);
+				RunCommand ("-environment-directory", Escape (startInfo.WorkingDirectory));
 				
 				// Set inferior arguments
 				if (!string.IsNullOrEmpty (startInfo.Arguments))
@@ -126,7 +126,7 @@ namespace MonoDevelop.Debugger.Gdb
 			}
 		}
 		
-		protected override void OnAttachToProcess (int processId)
+		protected override void OnAttachToProcess (long processId)
 		{
 			lock (gdbLock) {
 				StartGdb ();
@@ -184,7 +184,7 @@ namespace MonoDevelop.Debugger.Gdb
 				thread.Abort ();
 		}
 		
-		protected override void OnSetActiveThread (int processId, int threadId)
+		protected override void OnSetActiveThread (long processId, long threadId)
 		{
 			activeThread = threadId;
 		}
@@ -264,10 +264,31 @@ namespace MonoDevelop.Debugger.Gdb
 						if (!bp.BreakIfConditionChanges)
 							extraCmd += " -c " + bp.ConditionExpression;
 					}
-					RunCommand ("-environment-directory", Path.GetDirectoryName (bp.FileName));
-					GdbCommandResult res = RunCommand ("-break-insert", extraCmd.Trim (), bp.FileName + ":" + bp.Line);
-					if (CommandStatus.Error == res.Status)
-						res = RunCommand ("-break-insert", extraCmd.Trim (), Path.GetFileName (bp.FileName) + ":" + bp.Line);
+					
+					// Breakpoint locations must be double-quoted if files contain spaces.
+					// For example: -break-insert "\"C:/Documents and Settings/foo.c\":17"
+					
+					RunCommand ("-environment-directory", Escape (Path.GetDirectoryName (bp.FileName)));
+					GdbCommandResult res = null;
+					string errorMsg = null;
+					try {
+						res = RunCommand ("-break-insert", extraCmd.Trim (), Escape (Escape (bp.FileName) + ":" + bp.Line));
+					} catch (Exception ex) {
+						errorMsg = ex.Message;
+					}
+					
+					if (res == null) {
+						try {
+							res = RunCommand ("-break-insert", extraCmd.Trim (), Escape (Escape (Path.GetFileName (bp.FileName)) + ":" + bp.Line));
+						}
+						catch {
+						// Ignore
+						}
+					}
+					if (res == null) {
+						OnDebuggerOutput (true, "Could not set breakpoint: " + errorMsg);
+						return null;
+					}
 					int bh = res.GetObject ("bkpt").GetInt ("number");
 					if (!activate)
 						RunCommand ("-break-disable", bh.ToString ());
@@ -287,13 +308,13 @@ namespace MonoDevelop.Debugger.Gdb
 			
 			if (!string.IsNullOrEmpty (bp.ConditionExpression) && bp.BreakIfConditionChanges) {
 				// Update the condition expression
-				GdbCommandResult res = RunCommand ("-data-evaluate-expression", bp.ConditionExpression);
+				GdbCommandResult res = RunCommand ("-data-evaluate-expression", Escape (bp.ConditionExpression));
 				string val = res.GetValue ("value");
 				RunCommand ("-break-condition", handle.ToString (), "(" + bp.ConditionExpression + ") != " + val);
 			}
 			
 			if (bp.HitAction == HitAction.PrintExpression) {
-				GdbCommandResult res = RunCommand ("-data-evaluate-expression", bp.TraceExpression);
+				GdbCommandResult res = RunCommand ("-data-evaluate-expression", Escape (bp.TraceExpression));
 				string val = res.GetValue ("value");
 				OnDebuggerOutput (false, val + "\n");
 				NotifyBreakEventUpdate (handle, 0, val);
@@ -423,12 +444,12 @@ namespace MonoDevelop.Debugger.Gdb
 			RunCommand ("-exec-continue");
 		}
 		
-		protected override ThreadInfo[] OnGetThreads (int processId)
+		protected override ThreadInfo[] OnGetThreads (long processId)
 		{
 			List<ThreadInfo> list = new List<ThreadInfo> ();
 			ResultData data = RunCommand ("-thread-list-ids").GetObject ("thread-ids");
 			foreach (string id in data.GetAllValues ("thread-id"))
-				list.Add (GetThread (int.Parse (id)));
+				list.Add (GetThread (long.Parse (id)));
 			return list.ToArray ();
 		}
 		
@@ -438,12 +459,12 @@ namespace MonoDevelop.Debugger.Gdb
 			return new ProcessInfo [] { p };
 		}
 		
-		ThreadInfo GetThread (int id)
+		ThreadInfo GetThread (long id)
 		{
 			return new ThreadInfo (0, id, "Thread #" + id, null);
 		}
 		
-		protected override Backtrace OnGetThreadBacktrace (int processId, int threadId)
+		protected override Backtrace OnGetThreadBacktrace (long processId, long threadId)
 		{
 			ResultData data = SelectThread (threadId);
 			GdbCommandResult res = RunCommand ("-stack-info-depth");
@@ -485,7 +506,7 @@ namespace MonoDevelop.Debugger.Gdb
 			return lines.ToArray ();
 		}
 		
-		public ResultData SelectThread (int id)
+		public ResultData SelectThread (long id)
 		{
 			if (id == currentThread)
 				return null;
@@ -493,6 +514,18 @@ namespace MonoDevelop.Debugger.Gdb
 			return RunCommand ("-thread-select", id.ToString ());
 		}
 		
+		string Escape (string str)
+		{
+			if (str == null)
+				return null;
+			else if (str.IndexOf (' ') != -1 || str.IndexOf ('"') != -1) {
+				str = str.Replace ("\"", "\\\"");
+				return "\"" + str + "\"";
+			}
+			else
+				return str;
+		}
+		
 		public GdbCommandResult RunCommand (string command, params string[] args)
 		{
 			lock (gdbLock) {
@@ -502,7 +535,6 @@ namespace MonoDevelop.Debugger.Gdb
 					lock (eventLock) {
 						running = true;
 					}
-
 #if GDB_OUTPUT
 					Console.WriteLine ("gdb<: " + command + " " + string.Join (" ", args));
 #endif
@@ -576,8 +608,9 @@ namespace MonoDevelop.Debugger.Gdb
 					lock (eventLock) {
 						running = false;
 						ev = new GdbEvent (line);
-						if (ev.GetValue ("thread-id") != null)
-							currentThread = activeThread = ev.GetInt ("thread-id");
+						string ti = ev.GetValue ("thread-id");
+						if (ti != null && ti != "all")
+							currentThread = activeThread = int.Parse (ti);
 						Monitor.PulseAll (eventLock);
 						if (internalStop) {
 							internalStop = false;
